@@ -132,10 +132,9 @@ mod parsing {
                     }
                 };
 
-                return Some(Token {
-                    len: self.pos.checked_sub(start).expect("length underflow"),
-                    syntax,
-                });
+                let len = self.pos.saturating_sub(start);
+
+                return Some(Token { len, syntax });
             }
 
             None
@@ -187,17 +186,17 @@ mod parsing {
                 self.step();
             }
 
-            self.tree.insert_node_at(c, what);
+            self.tree.close_at(c, what);
             true
         }
 
         /// Bump the current input as the given syntax.
         pub(crate) fn bump_node(&mut self, what: Syntax) -> Result<()> {
-            self.tree.start_node(what);
+            self.tree.open(what);
             let tok = self.nth(0);
             self.step();
             self.tree.token(tok.syntax, tok.len);
-            self.tree.end_node()?;
+            self.tree.close()?;
             Ok(())
         }
 
@@ -289,7 +288,7 @@ mod grammar {
                         // an operator.
                         let c = p.tree.checkpoint();
                         p.advance_until(&[PLUS, MINUS]);
-                        p.tree.insert_node_at(c, ERROR);
+                        p.tree.close_at(c, ERROR);
                         continue;
                     }
                 }
@@ -361,9 +360,9 @@ mod eval {
         use EvalErrorKind::*;
 
         let mut it = tree.children().without_tokens();
-        let tree_span = tree.span();
+        let root_span = Span::new(0, source.len());
 
-        let initial = expect!(tree_span, it.next(), NUMBER, NUMBER);
+        let initial = expect!(root_span, it.next(), NUMBER, NUMBER);
         let span = initial.span();
         let mut n = source[span.range()]
             .parse::<i64>()
@@ -383,7 +382,7 @@ mod eval {
                 None => return Err(EvalError::new(span, ExpectedOperator)),
             };
 
-            let arg = expect!(tree_span, it.next(), NUMBER, NUMBER);
+            let arg = expect!(root_span, it.next(), NUMBER, NUMBER);
             let span = arg.span();
             let arg = source[arg.span().range()]
                 .parse::<i64>()
