@@ -6,19 +6,19 @@ fn main() -> Result<()> {
 
     let c = b.checkpoint();
 
-    b.start_node(1);
+    b.open(1);
     b.token(2, 1);
     b.token(3, 1);
-    b.end_node()?;
+    b.close()?;
 
     b.token(4, 3);
 
-    b.start_node(5);
+    b.open(5);
     b.token(6, 3);
     b.token(7, 3);
-    b.end_node()?;
+    b.close()?;
 
-    b.insert_node_at(c, 0);
+    b.close_at(c, 0);
 
     let tree = b.build()?;
 
