@@ -3,43 +3,45 @@ use std::fmt;
 use std::mem;
 
 use crate::non_max::NonMaxUsize;
-use crate::tree::Kind;
-use crate::{Span, Tree};
+use crate::tree::Links;
+use crate::Kind;
+use crate::Span;
+use crate::Tree;
 
 /// The identifier of a node as returned by functions such as
-/// [TreeBuilder::start_node] or [TreeBuilder::token].
+/// [TreeBuilder::open] or [TreeBuilder::token].
 ///
-/// This can be used as a checkpoint in [TreeBuilder::insert_node_at], and a
+/// This can be used as a checkpoint in [TreeBuilder::close_at], and a
 /// checkpoint can be fetched up front from [TreeBuilder::checkpoint].
 #[derive(Debug, Clone, Copy)]
 #[repr(transparent)]
 pub struct Id(NonMaxUsize);
 
-/// Error raised by [TreeBuilder::end_node] if there currently is no node being
+/// Error raised by [TreeBuilder::close] if there currently is no node being
 /// built.
 ///
 /// # Examples
 ///
 /// ```
-/// use syntree::{EndNodeError, Span, TreeBuilder};
+/// use syntree::{CloseError, Span, TreeBuilder};
 ///
 /// # fn main() -> anyhow::Result<()> {
 /// let mut tree = TreeBuilder::new();
 ///
-/// tree.start_node("root");
-/// tree.end_node()?;
+/// tree.open("root");
+/// tree.close()?;
 ///
 /// // Syntax::Root and Syntax::Child is left open.
-/// assert!(matches!(tree.end_node(), Err(EndNodeError { .. })));
+/// assert!(matches!(tree.close(), Err(CloseError { .. })));
 /// # Ok(()) }
 /// ```
 #[derive(Debug)]
 #[non_exhaustive]
-pub struct EndNodeError;
+pub struct CloseError;
 
-impl Error for EndNodeError {}
+impl Error for CloseError {}
 
-impl fmt::Display for EndNodeError {
+impl fmt::Display for CloseError {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         write!(f, "no node being built")
     }
@@ -62,11 +64,11 @@ impl fmt::Display for EndNodeError {
 /// # fn main() -> anyhow::Result<()> {
 /// let mut tree = TreeBuilder::new();
 ///
-/// tree.start_node(Syntax::Number);
+/// tree.open(Syntax::Number);
 /// tree.token(Syntax::Lit, 3);
-/// tree.end_node()?;
+/// tree.close()?;
 ///
-/// tree.start_node(Syntax::Number);
+/// tree.open(Syntax::Number);
 ///
 /// // Syntax::Number is left open.
 /// assert!(matches!(tree.build(), Err(BuildError { .. })));
@@ -84,22 +86,10 @@ impl fmt::Display for BuildError {
     }
 }
 
-#[derive(Debug)]
-pub(crate) struct Links<T> {
-    /// The data associated with the node.
-    pub(crate) data: T,
-    /// The kind of the element.
-    pub(crate) kind: Kind,
-    /// Next sibling id.
-    pub(crate) next: Option<NonMaxUsize>,
-    /// The first child element.
-    pub(crate) first: Option<NonMaxUsize>,
-}
-
 /// A builder for a [Tree].
 ///
 /// This maintains a stack of nodes being built which has to be balanced with
-/// calls to [TreeBuilder::start_node] and [TreeBuilder::end_node].
+/// calls to [TreeBuilder::open] and [TreeBuilder::close].
 ///
 /// # Examples
 ///
@@ -115,12 +105,12 @@ pub(crate) struct Links<T> {
 /// # fn main() -> anyhow::Result<()> {
 /// let mut tree = TreeBuilder::new();
 ///
-/// tree.start_node(Syntax::Root);
-/// tree.start_node(Syntax::Child);
-/// tree.end_node()?;
-/// tree.start_node(Syntax::Child);
-/// tree.end_node()?;
-/// tree.end_node()?;
+/// tree.open(Syntax::Root);
+/// tree.open(Syntax::Child);
+/// tree.close()?;
+/// tree.open(Syntax::Child);
+/// tree.close()?;
+/// tree.close()?;
 ///
 /// let tree = tree.build()?;
 ///
@@ -137,12 +127,12 @@ pub(crate) struct Links<T> {
 #[derive(Debug)]
 pub struct TreeBuilder<T> {
     /// Data in the tree being built.
-    data: Vec<Links<T>>,
+    pub(crate) data: Vec<Links<T>>,
     /// Nodes currently being built.
     stack: Vec<NonMaxUsize>,
     /// The last sibling inserted.
     sibling: Option<NonMaxUsize>,
-    /// The current cursor to the source code being referenced.
+    /// The current cursor.
     cursor: usize,
 }
 
@@ -164,15 +154,15 @@ impl<T> TreeBuilder<T> {
     /// # fn main() -> anyhow::Result<()> {
     /// let mut tree = TreeBuilder::new();
     ///
-    /// tree.start_node(Syntax::Root);
+    /// tree.open(Syntax::Root);
     ///
-    /// tree.start_node(Syntax::Child);
-    /// tree.end_node()?;
+    /// tree.open(Syntax::Child);
+    /// tree.close()?;
     ///
-    /// tree.start_node(Syntax::Child);
-    /// tree.end_node()?;
+    /// tree.open(Syntax::Child);
+    /// tree.close()?;
     ///
-    /// tree.end_node()?;
+    /// tree.close()?;
     /// # Ok(()) }
     /// ```
     pub const fn new() -> Self {
@@ -188,7 +178,7 @@ impl<T> TreeBuilder<T> {
     ///
     /// This pushes a new link with the given type onto the stack which links
     /// itself onto the last sibling node that ben introduced either through
-    /// [TreeBuilder::end_node] or [TreeBuilder::insert_node_at].
+    /// [TreeBuilder::close] or [TreeBuilder::close_at].
     ///
     /// # Examples
     ///
@@ -204,29 +194,29 @@ impl<T> TreeBuilder<T> {
     /// # fn main() -> anyhow::Result<()> {
     /// let mut tree = TreeBuilder::new();
     ///
-    /// tree.start_node(Syntax::Root);
+    /// tree.open(Syntax::Root);
     ///
-    /// tree.start_node(Syntax::Child);
-    /// tree.end_node()?;
+    /// tree.open(Syntax::Child);
+    /// tree.close()?;
     ///
-    /// tree.start_node(Syntax::Child);
-    /// tree.end_node()?;
+    /// tree.open(Syntax::Child);
+    /// tree.close()?;
     ///
-    /// tree.end_node()?;
+    /// tree.close()?;
     /// # Ok(()) }
     /// ```
-    pub fn start_node(&mut self, data: T) -> Id {
-        let id = self.insert(data, Kind::Node);
+    pub fn open(&mut self, data: T) -> Id {
+        let id = self.insert(data, Kind::Node, Span::point(self.cursor));
         self.stack.push(id);
         Id(id)
     }
 
     /// End a node being built. This call must be balanced with a prior call to
-    /// [TreeBuilder::start_node] and if its not will result in an
-    /// [EndNodeError].
+    /// [TreeBuilder::open] and if its not will result in an
+    /// [CloseError].
     ///
     /// This will pop a value of the stack, and set that value as the next
-    /// sibling which will be used with [TreeBuilder::start_node].
+    /// sibling which will be used with [TreeBuilder::open].
     ///
     /// # Examples
     ///
@@ -242,24 +232,35 @@ impl<T> TreeBuilder<T> {
     /// # fn main() -> anyhow::Result<()> {
     /// let mut tree = TreeBuilder::new();
     ///
-    /// tree.start_node(Syntax::Root);
+    /// tree.open(Syntax::Root);
     ///
-    /// tree.start_node(Syntax::Child);
-    /// tree.end_node()?;
+    /// tree.open(Syntax::Child);
+    /// tree.close()?;
     ///
-    /// tree.start_node(Syntax::Child);
-    /// tree.end_node()?;
+    /// tree.open(Syntax::Child);
+    /// tree.close()?;
     ///
-    /// tree.end_node()?;
+    /// tree.close()?;
     /// # Ok(()) }
     /// ```
-    pub fn end_node(&mut self) -> Result<(), EndNodeError> {
+    pub fn close(&mut self) -> Result<(), CloseError> {
         let head = match self.stack.pop() {
             Some(head) => head,
-            None => return Err(EndNodeError),
+            None => return Err(CloseError),
         };
 
         self.sibling = Some(head);
+
+        if let Some(parent) = self.stack.last().copied() {
+            if let Some(node) = self.data.get_mut(head.get()) {
+                let end = node.span.end;
+
+                if let Some(parent) = self.data.get_mut(parent.get()) {
+                    parent.span.end = end;
+                }
+            }
+        }
+
         Ok(())
     }
 
@@ -282,16 +283,17 @@ impl<T> TreeBuilder<T> {
     /// # fn main() -> anyhow::Result<()> {
     /// let mut tree = TreeBuilder::new();
     ///
-    /// tree.start_node(Syntax::Child);
+    /// tree.open(Syntax::Child);
     /// tree.token(Syntax::Number, 3);
-    /// tree.end_node()?;
+    /// tree.close()?;
     ///
     /// # Ok(()) }
     /// ```
     pub fn token(&mut self, data: T, len: usize) -> Id {
         let start = self.cursor;
-        self.cursor = self.cursor.checked_add(len).expect("length overflow");
-        let id = self.insert(data, Kind::Token(Span::new(start, self.cursor)));
+        let end = self.cursor + len;
+        self.cursor = end;
+        let id = self.insert(data, Kind::Token, Span::new(start, end));
         self.sibling = Some(id);
         Id(id)
     }
@@ -322,18 +324,18 @@ impl<T> TreeBuilder<T> {
     ///
     /// let c = b.checkpoint();
     ///
-    /// b.start_node(Syntax::Number);
+    /// b.open(Syntax::Number);
     /// b.token(Syntax::Lit, 1);
-    /// b.end_node()?;
+    /// b.close()?;
     ///
     /// b.token(Syntax::Whitespace, 3);
     ///
-    /// b.start_node(Syntax::Number);
+    /// b.open(Syntax::Number);
     /// b.token(Syntax::Lit, 2);
     /// b.token(Syntax::Lit, 2);
-    /// b.end_node()?;
+    /// b.close()?;
     ///
-    /// b.insert_node_at(c, Syntax::Root);
+    /// b.close_at(c, Syntax::Root);
     ///
     /// let tree = b.build()?;
     ///
@@ -353,25 +355,25 @@ impl<T> TreeBuilder<T> {
     /// This causes the node specified at `id` to become the previous sibling
     /// node. If `id` refers to a node that hasn't been allocated yet (through
     /// [TreeBuilder::checkpoint]), this call corresponds exactly to
-    /// [TreeBuilder::start_node] that is immediately closed with
-    /// [TreeBuilder::end_node].
+    /// [TreeBuilder::open] that is immediately closed with
+    /// [TreeBuilder::close].
     ///
     /// ```
     /// use syntree::TreeBuilder;
     ///
     /// # fn main() -> anyhow::Result<()> {
     /// let mut a = TreeBuilder::<u32>::new();
-    /// a.start_node(0);
+    /// a.open(0);
     /// let c = a.checkpoint();
-    /// a.insert_node_at(c, 1);
-    /// a.end_node()?;
+    /// a.close_at(c, 1);
+    /// a.close()?;
     /// let a = a.build()?;
     ///
     /// let mut b = TreeBuilder::<u32>::new();
-    /// b.start_node(0);
+    /// b.open(0);
     /// let c = b.checkpoint();
-    /// b.insert_node_at(c, 1);
-    /// b.end_node()?;
+    /// b.close_at(c, 1);
+    /// b.close()?;
     /// let b = b.build()?;
     ///
     /// assert_eq!(a, b);
@@ -379,8 +381,8 @@ impl<T> TreeBuilder<T> {
     /// ```
     ///
     /// Note that this does not modify or try to balance the stack, so the last
-    /// item pushed using [TreeBuilder::start_node] will still be the one popped
-    /// through [TreeBuilder::end_node].
+    /// item pushed using [TreeBuilder::open] will still be the one popped
+    /// through [TreeBuilder::close].
     ///
     /// # Examples
     ///
@@ -389,39 +391,49 @@ impl<T> TreeBuilder<T> {
     ///
     /// #[derive(Debug, Clone, Copy, PartialEq, Eq, Hash)]
     /// enum Syntax {
-    ///     Root,
-    ///     Number,
-    ///     Lit,
-    ///     Whitespace,
+    ///     ROOT,
+    ///     NUMBER,
+    ///     LIT,
+    ///     WHITESPACE,
     /// }
     ///
+    /// use Syntax::*;
+    ///
     /// # fn main() -> anyhow::Result<()> {
     /// let mut b = TreeBuilder::new();
     ///
     /// let c = b.checkpoint();
     ///
-    /// b.start_node(Syntax::Number);
-    /// b.token(Syntax::Lit, 1);
-    /// b.end_node()?;
+    /// b.open(NUMBER);
+    /// b.token(LIT, 3);
+    /// b.close()?;
     ///
-    /// b.token(Syntax::Whitespace, 3);
+    /// b.token(WHITESPACE, 1);
     ///
-    /// b.start_node(Syntax::Number);
-    /// b.token(Syntax::Lit, 2);
-    /// b.token(Syntax::Lit, 2);
-    /// b.end_node()?;
+    /// b.open(NUMBER);
+    /// b.token(LIT, 2);
+    /// b.close()?;
     ///
-    /// b.insert_node_at(c, Syntax::Root);
+    /// b.close_at(c, ROOT);
     ///
     /// let tree = b.build()?;
     ///
-    /// let root = tree.first().unwrap();
-    /// assert_eq!(*root.data(), Syntax::Root);
-    /// assert_eq!(root.children().count(), 3);
-    /// assert_eq!(root.children().without_tokens().count(), 2);
+    /// let expected = syntree::tree! {
+    ///     ROOT => {
+    ///         NUMBER => {
+    ///             (LIT, 3)
+    ///         },
+    ///         (WHITESPACE, 1),
+    ///         NUMBER => {
+    ///             (LIT, 2)
+    ///         },
+    ///     }
+    /// };
+    ///
+    /// assert_eq!(tree, expected);
     /// # Ok(()) }
     /// ```
-    pub fn insert_node_at(&mut self, id: Id, data: T) -> Id {
+    pub fn close_at(&mut self, id: Id, data: T) -> Id {
         // With the layout of this data structure this is a fairly simple
         // operation.
         let child = NonMaxUsize::new(self.data.len()).expect("ran out of ids");
@@ -429,7 +441,7 @@ impl<T> TreeBuilder<T> {
         let links = match self.data.get_mut(id.0.get()) {
             Some(links) => links,
             None => {
-                let id = self.insert(data, Kind::Node);
+                let id = self.insert(data, Kind::Node, Span::point(self.cursor));
                 self.sibling = Some(id);
                 return Id(id);
             }
@@ -442,9 +454,26 @@ impl<T> TreeBuilder<T> {
                 kind: Kind::Node,
                 next: None,
                 first: Some(child),
+                span: links.span,
             },
         );
 
+        // Adjust span to encapsulate all children.
+        let mut span = links.span;
+
+        if let Some(mut cur) = removed.next.and_then(|id| self.data.get(id.get())) {
+            span = span.join(cur.span);
+
+            while let Some(next) = cur.next.and_then(|id| self.data.get(id.get())) {
+                span = span.join(next.span);
+                cur = next;
+            }
+
+            if let Some(node) = self.data.get_mut(id.0.get()) {
+                node.span = span;
+            }
+        }
+
         self.data.push(removed);
 
         // The current sibling is the newly replaced node in the tree.
@@ -471,11 +500,11 @@ impl<T> TreeBuilder<T> {
     /// # fn main() -> anyhow::Result<()> {
     /// let mut tree = TreeBuilder::new();
     ///
-    /// tree.start_node(Syntax::Child);
+    /// tree.open(Syntax::Child);
     /// tree.token(Syntax::Number, 3);
-    /// tree.end_node()?;
-    /// tree.start_node(Syntax::Child);
-    /// tree.end_node()?;
+    /// tree.close()?;
+    /// tree.open(Syntax::Child);
+    /// tree.close()?;
     ///
     /// let tree = tree.build()?;
     ///
@@ -504,17 +533,17 @@ impl<T> TreeBuilder<T> {
     /// # fn main() -> anyhow::Result<()> {
     /// let mut tree = TreeBuilder::new();
     ///
-    /// tree.start_node(Syntax::Number);
+    /// tree.open(Syntax::Number);
     /// tree.token(Syntax::Lit, 3);
-    /// tree.end_node()?;
+    /// tree.close()?;
     ///
-    /// tree.start_node(Syntax::Number);
+    /// tree.open(Syntax::Number);
     ///
     /// // Syntax::Number is left open.
     /// assert!(matches!(tree.build(), Err(BuildError { .. })));
     /// # Ok(()) }
     /// ```
-    pub fn build(&self) -> Result<Tree<T>, BuildError>
+    pub fn build(self) -> Result<Tree<T>, BuildError>
     where
         T: Clone,
     {
@@ -522,16 +551,11 @@ impl<T> TreeBuilder<T> {
             return Err(BuildError);
         }
 
-        Ok(crate::convert::builder_to_tree(self))
-    }
-
-    /// Get the links corresponding to the given id.
-    pub(crate) fn get(&self, id: usize) -> Option<&Links<T>> {
-        self.data.get(id)
+        Ok(Tree::new(self.data.into()))
     }
 
     /// Insert a new node.
-    fn insert(&mut self, data: T, kind: Kind) -> NonMaxUsize {
+    fn insert(&mut self, data: T, kind: Kind, span: Span) -> NonMaxUsize {
         let new = NonMaxUsize::new(self.data.len()).expect("ran out of ids");
 
         let prev = std::mem::replace(&mut self.sibling, None);
@@ -542,16 +566,19 @@ impl<T> TreeBuilder<T> {
             kind,
             next: None,
             first: None,
+            span,
         });
 
-        if let Some(prev) = prev.and_then(|id| self.data.get_mut(id.get())) {
-            prev.next = Some(new);
+        if let Some(node) = prev.and_then(|id| self.data.get_mut(id.get())) {
+            node.next = Some(new);
         }
 
-        if let Some(n) = parent.and_then(|id| self.data.get_mut(id.get())) {
-            if n.first.is_none() {
-                n.first = Some(new);
+        if let Some(node) = parent.and_then(|id| self.data.get_mut(id.get())) {
+            if node.first.is_none() {
+                node.first = Some(new);
             }
+
+            node.span.end = span.end;
         }
 
         new
