@@ -1,10 +1,8 @@
 use std::fmt;
 
 use crate::non_max::NonMaxUsize;
-use crate::{
-    tree::{Kind, Links},
-    Children, Span, Walk,
-};
+use crate::tree::{Kind, Links};
+use crate::{Children, Span, Walk};
 
 /// A node in the tree.
 pub struct Node<'a, T> {
@@ -38,17 +36,17 @@ impl<'a, T> Node<'a, T> {
     /// # fn main() -> anyhow::Result<()> {
     /// let mut tree = TreeBuilder::new();
     ///
-    /// tree.start_node("root");
+    /// tree.open("root");
     ///
-    /// tree.start_node("number");
+    /// tree.open("number");
     /// tree.token("number", 5);
-    /// tree.end_node()?;
+    /// tree.close()?;
     ///
-    /// tree.start_node("ident");
+    /// tree.open("ident");
     /// tree.token("ident", 2);
-    /// tree.end_node()?;
+    /// tree.close()?;
     ///
-    /// tree.end_node()?;
+    /// tree.close()?;
     ///
     /// let tree = tree.build()?;
     ///
@@ -58,11 +56,7 @@ impl<'a, T> Node<'a, T> {
     /// # Ok(()) }
     /// ```
     pub fn span(&self) -> Span {
-        if let Some(span) = self.children().span() {
-            span
-        } else {
-            Span::new(0, usize::MAX)
-        }
+        self.node.span
     }
 
     /// Check if the current node is empty. In that it doesn't have any
@@ -74,29 +68,23 @@ impl<'a, T> Node<'a, T> {
     /// # fn main() -> anyhow::Result<()> {
     /// let mut tree = TreeBuilder::new();
     ///
-    /// tree.start_node("first");
-    /// tree.end_node()?;
+    /// tree.open("first");
+    /// tree.close()?;
     ///
-    /// tree.start_node("last");
+    /// tree.open("last");
     /// tree.token("token", 5);
-    /// tree.end_node()?;
+    /// tree.close()?;
     ///
     /// let tree = tree.build()?;
     ///
     /// let first = tree.first().expect("expected first root node");
-    /// let last = tree.last().expect("expected last root node");
+    /// let last = first.next().expect("expected last root node");
     ///
     /// assert!(first.is_empty());
     /// assert!(!last.is_empty());
     /// # Ok(()) }
     /// ```
     pub fn is_empty(&self) -> bool {
-        if cfg!(debug_assertions) {
-            if self.node.first.is_none() {
-                debug_assert_eq!(self.node.last, None);
-            }
-        }
-
         self.node.first.is_none()
     }
 
@@ -108,13 +96,13 @@ impl<'a, T> Node<'a, T> {
     /// # fn main() -> anyhow::Result<()> {
     /// let mut tree = TreeBuilder::new();
     ///
-    /// tree.start_node("root1");
-    /// tree.start_node("child1");
-    /// tree.end_node()?;
+    /// tree.open("root1");
+    /// tree.open("child1");
+    /// tree.close()?;
     ///
-    /// tree.start_node("child2");
-    /// tree.end_node()?;
-    /// tree.end_node()?;
+    /// tree.open("child2");
+    /// tree.close()?;
+    /// tree.close()?;
     ///
     /// let tree = tree.build()?;
     /// let root = tree.first().expect("expected root node");
@@ -130,11 +118,11 @@ impl<'a, T> Node<'a, T> {
         Children {
             tree: self.tree,
             start: self.node.first,
-            end: self.node.last,
         }
     }
 
-    /// Walk the rest of the tree forwards in a depth-first fashion.
+    /// Walk the subtree forward starting with the first child of the current
+    /// node.
     ///
     /// # Examples
     ///
@@ -158,31 +146,8 @@ impl<'a, T> Node<'a, T> {
     /// assert_eq!(nodes, vec!["c1", "c2", "c3", "c4", "c5", "c6"]);
     /// # Ok(()) }
     /// ```
-    ///
-    /// Iterate backwards:
-    ///
-    /// ```
-    /// # fn main() -> anyhow::Result<()> {
-    /// let tree = syntree::tree! {
-    ///     "root" => {
-    ///         "c1" => {
-    ///             "c2",
-    ///             "c3",
-    ///             "c4",
-    ///         },
-    ///         "c5",
-    ///         "c6"
-    ///     }
-    /// };
-    ///
-    /// let root = tree.first().expect("expected root node");
-    ///
-    /// let nodes = root.walk().rev().map(|n| *n.data()).collect::<Vec<_>>();
-    /// assert_eq!(nodes, vec!["c6", "c5", "c4", "c3", "c2", "c1"]);
-    /// # Ok(()) }
-    /// ```
     pub fn walk(&self) -> Walk<'a, T> {
-        self.children().walk()
+        Walk::new(self.tree, self.node.first, self.node.next)
     }
 
     /// Get the first child node.
@@ -193,15 +158,15 @@ impl<'a, T> Node<'a, T> {
     /// # fn main() -> anyhow::Result<()> {
     /// let mut tree = TreeBuilder::new();
     ///
-    /// tree.start_node("root1");
+    /// tree.open("root1");
     ///
-    /// tree.start_node("child1");
-    /// tree.end_node()?;
+    /// tree.open("child1");
+    /// tree.close()?;
     ///
-    /// tree.start_node("child2");
-    /// tree.end_node()?;
+    /// tree.open("child2");
+    /// tree.close()?;
     ///
-    /// tree.end_node()?;
+    /// tree.close()?;
     ///
     /// let tree = tree.build()?;
     /// let root = tree.first().expect("expected root node");
@@ -213,34 +178,6 @@ impl<'a, T> Node<'a, T> {
         self.node_at(self.node.first)
     }
 
-    /// Get the last child node.
-    ///
-    /// ```
-    /// use syntree::TreeBuilder;
-    ///
-    /// # fn main() -> anyhow::Result<()> {
-    /// let mut tree = TreeBuilder::new();
-    ///
-    /// tree.start_node("root1");
-    ///
-    /// tree.start_node("child1");
-    /// tree.end_node()?;
-    ///
-    /// tree.start_node("child2");
-    /// tree.end_node()?;
-    ///
-    /// tree.end_node()?;
-    ///
-    /// let tree = tree.build()?;
-    /// let root = tree.first().expect("expected root node");
-    ///
-    /// assert_eq!(root.last().map(|n| *n.data()), Some("child2"));
-    /// # Ok(()) }
-    /// ```
-    pub fn last(&self) -> Option<Node<'a, T>> {
-        self.node_at(self.node.last)
-    }
-
     /// Get the next sibling.
     ///
     /// ```
@@ -249,18 +186,18 @@ impl<'a, T> Node<'a, T> {
     /// # fn main() -> anyhow::Result<()> {
     /// let mut tree = TreeBuilder::new();
     ///
-    /// tree.start_node("root1");
+    /// tree.open("root1");
     ///
-    /// tree.start_node("child1");
-    /// tree.end_node()?;
+    /// tree.open("child1");
+    /// tree.close()?;
     ///
-    /// tree.start_node("child2");
-    /// tree.end_node()?;
+    /// tree.open("child2");
+    /// tree.close()?;
     ///
-    /// tree.start_node("child3");
-    /// tree.end_node()?;
+    /// tree.open("child3");
+    /// tree.close()?;
     ///
-    /// tree.end_node()?;
+    /// tree.close()?;
     ///
     /// let tree = tree.build()?;
     /// let root = tree.first().expect("expected root node");
@@ -274,39 +211,6 @@ impl<'a, T> Node<'a, T> {
         self.node_at(self.node.next)
     }
 
-    /// Get the previous sibling.
-    ///
-    /// ```
-    /// use syntree::TreeBuilder;
-    ///
-    /// # fn main() -> anyhow::Result<()> {
-    /// let mut tree = TreeBuilder::new();
-    ///
-    /// tree.start_node("root1");
-    ///
-    /// tree.start_node("child1");
-    /// tree.end_node()?;
-    ///
-    /// tree.start_node("child2");
-    /// tree.end_node()?;
-    ///
-    /// tree.start_node("child3");
-    /// tree.end_node()?;
-    ///
-    /// tree.end_node()?;
-    ///
-    /// let tree = tree.build()?;
-    /// let root = tree.first().expect("expected root node");
-    ///
-    /// let child = root.last().expect("expected last node");
-    /// assert_eq!(*child.data(), "child3");
-    /// assert_eq!(child.prev().map(|n| *n.data()), Some("child2"));
-    /// # Ok(()) }
-    /// ```
-    pub fn prev(&self) -> Option<Node<'a, T>> {
-        self.node_at(self.node.prev)
-    }
-
     fn node_at(&self, index: Option<NonMaxUsize>) -> Option<Node<'a, T>> {
         let cur = self.tree.get(index?.get())?;
 
@@ -336,3 +240,14 @@ impl<'a, T> Clone for Node<'a, T> {
 }
 
 impl<'a, T> Copy for Node<'a, T> {}
+
+impl<'a, T> PartialEq for Node<'a, T>
+where
+    T: PartialEq,
+{
+    fn eq(&self, other: &Self) -> bool {
+        self.node.data == other.node.data && self.node.kind == other.node.kind
+    }
+}
+
+impl<'a, T> Eq for Node<'a, T> where T: Eq {}
